import React, { useState } from 'react';
import { VaultFile, VaultFileObjectKey } from '../utilities/vault';
import {
  generateAxiomLink,
  getDisplayPercent,
  invokeClientDownload,
  IS_BROWSER,
} from '../utilities/helpers';
import { Table } from './layout/Table/Table';
import cn from 'classnames';
import {
  handleFileDownload,
  IPFSDownloadType,
  STORE_TYPES_MAP,
  StoreType,
} from '../utilities/files';
import { ClipboardCopyIcon, DownloadIcon } from '@heroicons/react/outline';
import { useCurrentDownloads } from '../context/CurrentDownloadsContext';

type VaultTableRowProps = {
  file: VaultFile;
  index: number;
  realIndex: number;
  downloadType: IPFSDownloadType;
};

export const VaultTableRow: React.FC<VaultTableRowProps> = ({
  file,
  index,
  realIndex,
  downloadType,
}) => {
  const { downloads, setIsDownloading } = useCurrentDownloads();

  const [progress, setProgress] = useState<number>();
  const { storeType } = file[VaultFileObjectKey.FileOptions];
  const encryption = file[VaultFileObjectKey.Encryption];

  const axiomLink = generateAxiomLink(file[VaultFileObjectKey.CID], {
    ...encryption,
    storeType,
  });

  const isDownloading = downloads?.[axiomLink];

  const handleSingleDownload = (axiomLink: string, vaultFile: VaultFile) => {
    setIsDownloading(axiomLink, true);

    void handleFileDownload(axiomLink, downloadType, vaultFile, {
      onCompleted: (file) => {
        invokeClientDownload(file);
        setIsDownloading(axiomLink, false);
        setProgress(undefined);
      },
      onProgress: (percent) => setProgress(getDisplayPercent(percent)),
      onError: () => {
        setIsDownloading(axiomLink, false);
        setProgress(undefined);
      },
    });
  };

  return (
    <Table.Row key={`${file[VaultFileObjectKey.CID]}${realIndex}`}>
      <Table.Col as="th">{index + 1}</Table.Col>
      <Table.Col>
        <div className="text-sm whitespace-pre-wrap">
          {file[VaultFileObjectKey.Filename]}
        </div>
      </Table.Col>
      <Table.Col>
        <span
          className={cn(
            'badge',
            storeType === StoreType.RAW ? 'badge-ghost' : 'badge-info',
          )}
        >
          {STORE_TYPES_MAP[storeType]}
        </span>
      </Table.Col>
      <Table.Col>
        {encryption && <div className="font-bold text-center">Yes</div>}
      </Table.Col>
      <Table.Col>
        <div
          className={cn('flex space-x-1.5 transition-all', {
            'pb-3': isDownloading,
          })}
        >
          <button
            className={cn('btn btn-sm btn-ghost')}
            disabled={isDownloading}
            onClick={() => handleSingleDownload(axiomLink, file)}
          >
            <DownloadIcon width={24} />
          </button>
          <button
            className="btn btn-sm btn-ghost"
            onClick={() => {
              if (IS_BROWSER) {
                void navigator.clipboard.writeText(axiomLink);
              }
            }}
          >
            <ClipboardCopyIcon width={24} />
          </button>
        </div>
        {isDownloading && (
          <progress
            className="absolute progress progress-primary w-28 h-1 transform -translate-y-1 transition-all"
            value={
              downloadType === IPFSDownloadType.Node ? undefined : progress
            }
            max={100}
          />
        )}
      </Table.Col>
    </Table.Row>
  );
};
