import React, { ReactNode } from 'react';
import { ComponentBase, WithClassName } from '../../../types/components';
import cn from 'classnames';
import { DynamicArrayInput } from '../../../utilities/helpers';
import {
  DEFAULT_PAGINATION_PROPS,
  PaginationProps,
} from '../../../utilities/pagination';
import { usePagination } from '../../../hooks/usePagination';
import { Pagination } from '../../controls/Pagination';

type TableProps<T> = WithClassName & {
  head: ReactNode[];
  items: DynamicArrayInput<T>;
  render: (item: T, realIndex: number, index: number) => ReactNode;
  pagination?: PaginationProps;
  emptyPlaceholder?: string;
};

export const Table = <T extends any>({
  className,
  head,
  items,
  pagination,
  render,
  emptyPlaceholder,
}: TableProps<T>) => {
  const [iterableItems, { paginationData, previousPage, nextPage, setPage }] =
    usePagination<T>(items, pagination ?? DEFAULT_PAGINATION_PROPS);

  return (
    <div>
      <div className={cn(className, 'overflow-x-auto w-full')}>
        <table className="table w-full">
          <thead>
            <tr>
              {head.map((cell, index) => (
                <th key={`${cell?.toString()}${index}`}>
                  <label>{cell}</label>
                </th>
              ))}
            </tr>
          </thead>
          <tbody>
            {iterableItems.length === 0 && emptyPlaceholder ? (
              <tr>
                <td colSpan={head.length} className="text-center">
                  {emptyPlaceholder}
                </td>
              </tr>
            ) : (
              iterableItems.map((item, index) =>
                render(
                  item,
                  (paginationData.page - 1) * paginationData.pageSize + index,
                  index,
                ),
              )
            )}
          </tbody>
        </table>
      </div>
      <div className="grid justify-center mt-6">
        <Pagination
          controls={{
            paginationData,
            previousPage,
            nextPage,
            setPage,
          }}
        />
      </div>
    </div>
  );
};

type TableColProps = ComponentBase & {
  as?: string;
};

const TableCol: React.FC<TableColProps> = ({
  as = 'td',
  className,
  children,
}) => React.createElement(as, { className }, children);

Table.Col = TableCol;

const TableRow: React.FC<ComponentBase> = ({ className, children }) => (
  <tr className={cn(className, 'hover')}>{children}</tr>
);

Table.Row = TableRow;
