import React, { createContext, useContext, useEffect, useState } from 'react';
import { WithChildren } from '../types/components';

export type Theme = 'light' | 'dark';

export const THEMES: Theme[] = ['light', 'dark'];
export const THEME_LOCAL_STORAGE_KEY = 'appTheme';
const MATCH_MEDIA = '(prefers-color-scheme: dark)';

export type ThemeContextProps = {
  theme: Theme;
  toggleTheme: (theme?: Theme) => void;
};

export const ThemeContext = createContext<ThemeContextProps>({
  theme: 'light',
  toggleTheme: () => {},
});

export const useTheme = () => useContext(ThemeContext);

export const ThemeContextProvider: React.FC<WithChildren> = ({ children }) => {
  const [_theme, setTheme] = useState<Theme>('light');

  useEffect(() => {
    let hasPreference = false;
    const theme = localStorage.getItem(THEME_LOCAL_STORAGE_KEY);

    if (theme && THEMES.includes(theme as Theme)) {
      setTheme(theme as Theme);
      hasPreference = true;
    }

    const handler = ({ matches }: MediaQueryListEvent) => {
      if (!hasPreference) {
        setTheme(matches ? 'dark' : 'light');
      }
    };

    const matchMedia = window.matchMedia(MATCH_MEDIA);
    matchMedia.addEventListener('change', handler);

    return () => matchMedia.removeEventListener('change', handler);
  }, []);

  const toggleTheme = (theme?: Theme) => {
    localStorage.setItem(
      THEME_LOCAL_STORAGE_KEY,
      theme || _theme === 'light' ? 'dark' : 'light',
    );

    if (theme) {
      setTheme(theme);
    } else {
      setTheme((prev) => (prev === 'light' ? 'dark' : 'light'));
    }
  };

  return (
    <ThemeContext.Provider value={{ theme: _theme, toggleTheme }}>
      {children}
    </ThemeContext.Provider>
  );
};
