import JSZip from 'jszip';

export const AXIOM_METADATA_FILENAME = 'axiom.json';

export enum WrappedFileMetadataKey {
  Filename = 'fn',
  FileType = 'ft',
  LastModified = 'm',
  Timestamp = 't',
  Encrypted = 'e',
  ObfuscatedFilename = 'of',
}

export type WrappedFileMetadata = {
  [WrappedFileMetadataKey.Filename]: string;
  [WrappedFileMetadataKey.FileType]: string;
  [WrappedFileMetadataKey.LastModified]: number;
  [WrappedFileMetadataKey.Timestamp]: number;
  [WrappedFileMetadataKey.Encrypted]: boolean;
  [WrappedFileMetadataKey.ObfuscatedFilename]: boolean;
};

type WrappedFileOptions = {
  encrypted?: boolean;
  obfuscateFilename?: boolean;
};

export const wrapFile = async (
  file: File,
  { encrypted = false, obfuscateFilename = false }: WrappedFileOptions = {},
): Promise<File> => {
  const zip = new JSZip();
  const wrapper = zip.folder(crypto.randomUUID());

  const filename = obfuscateFilename ? crypto.randomUUID() : file.name;

  wrapper?.file(filename, await file.arrayBuffer());

  const metadata: WrappedFileMetadata = {
    [WrappedFileMetadataKey.Filename]: filename,
    [WrappedFileMetadataKey.FileType]: file.type,
    [WrappedFileMetadataKey.LastModified]: file.lastModified,
    [WrappedFileMetadataKey.Timestamp]: new Date().getTime(),
    [WrappedFileMetadataKey.Encrypted]: encrypted,
    [WrappedFileMetadataKey.ObfuscatedFilename]: obfuscateFilename,
  };

  wrapper?.file(AXIOM_METADATA_FILENAME, JSON.stringify(metadata));

  return new Promise((resolve, reject) => {
    wrapper
      ?.generateAsync({ type: 'blob' })
      .then((res) => {
        resolve(
          new File([res], `${crypto.randomUUID()}.zip`, {
            type: 'application/zip',
          }),
        );
      })
      .catch(reject);
  });
};

export const getWrapperMetadata = async (
  zipFile: File,
): Promise<WrappedFileMetadata> => {
  const zip = JSZip();

  return new Promise((resolve, reject) => {
    zip
      .loadAsync(zipFile)
      .then((unzip) => {
        unzip
          .file(AXIOM_METADATA_FILENAME)
          ?.async('string')
          .then((serializedMetadata) => {
            resolve(JSON.parse(serializedMetadata) as WrappedFileMetadata);
          });
      })
      .catch(reject);
  });
};

export const unwrapFile = async (zipFile: File): Promise<File> => {
  const zip = JSZip();

  return new Promise((resolve, reject) => {
    zip.loadAsync(zipFile).then(async (unzip) => {
      const metadata = await getWrapperMetadata(zipFile);

      unzip
        .file(metadata[WrappedFileMetadataKey.Filename])
        ?.async('blob')
        .then((blob) => {
          resolve(
            new File([blob], metadata[WrappedFileMetadataKey.Filename], {
              type: metadata[WrappedFileMetadataKey.FileType],
              lastModified: metadata[WrappedFileMetadataKey.LastModified],
            }),
          );
        })
        .catch(reject);
    });
  });
};
