import { decryptFile, encryptFile } from './encryption';
import { unwrapFile, wrapFile } from './file-wrapper';
import ky from 'ky';
import { fromBase64, IPFS_GATEWAY } from './helpers';
import { VaultFile, VaultFileObjectKey } from './vault';
import { queryIPFS } from './ipfs';

export enum StoreType {
  RAW = 'raw',
  WRAPPED = 'wrapper',
}

export const STORE_TYPES_MAP: Record<StoreType, string> = {
  [StoreType.RAW]: 'Raw',
  [StoreType.WRAPPED]: 'Wrapped File',
};

export enum AxiomLinkQuery {
  CID = 'cid',
  EncryptionKey = 'k',
  IV = 'i',
  StoreType = 's',
}

export type AxiomLinkMetadataSerialized = {
  [AxiomLinkQuery.CID]: string;
  [AxiomLinkQuery.EncryptionKey]?: string;
  [AxiomLinkQuery.IV]?: string;
  [AxiomLinkQuery.StoreType]: StoreType;
};

export type FileOptions = {
  storeType: StoreType;
  encrypt?: boolean;
  obfuscateFilename?: boolean;
};

type ProcessedFile = {
  file: File;
} & ProcessedFileMetadata;

export type ProcessedFileMetadata = {
  iv?: string;
  key?: JsonWebKey;
  storeType: StoreType;
};

export const processFile = async (
  file: File,
  { storeType, encrypt, obfuscateFilename }: FileOptions,
): Promise<ProcessedFile> => {
  if (storeType === StoreType.RAW) {
    if (encrypt) {
      const { data, iv, key } = await encryptFile(file);

      return {
        file: data,
        iv,
        key,
        storeType,
      };
    }

    return { file: file, storeType };
  }

  if (encrypt) {
    const { data, iv, key } = await encryptFile(file);

    const wrapper = await wrapFile(data, {
      encrypted: true,
      obfuscateFilename: obfuscateFilename,
    });

    return {
      file: wrapper,
      iv,
      key,
      storeType,
    };
  }

  const wrapper = await wrapFile(file, {
    encrypted: false,
    obfuscateFilename: obfuscateFilename,
  });

  return {
    file: wrapper,
    storeType,
  };
};

export const reconstructVaultFile = (
  blob: Blob,
  metadata?: VaultFile,
): File => {
  return new File([blob], metadata?.[VaultFileObjectKey.Filename] ?? '', {
    type: metadata?.[VaultFileObjectKey.FileType],
    lastModified: metadata?.[VaultFileObjectKey.LastModified],
  });
};

export enum DownloadInputType {
  CID,
  Axiom,
}

type ParseDownloadInputReturn = {
  type: DownloadInputType;
  cid: string;
};

export const parseDownloadInput = async (
  value: string,
): Promise<ParseDownloadInputReturn> => {
  return new Promise((resolve, reject) => {
    if (value.length === 0) {
      reject();
    }

    let type: DownloadInputType;
    let cid: string = '';

    try {
      const url = new URL(value);
      cid = url.searchParams.get('cid') ?? '';
      type = DownloadInputType.Axiom;
    } catch (_) {
      cid = value;
      type = DownloadInputType.CID;
    }

    resolve({ type, cid });
  });
};

type HandleFileDownloadCallbacks = {
  onProgress?: (percent: number) => void;
  onCompleted: (file: File) => void;
  onError?: () => void;
};

export enum IPFSDownloadType {
  Gateway,
  Node,
}

export const IPFS_DOWNLOAD_TYPES_MAP: {
  type: IPFSDownloadType;
  label: string;
}[] = [
  { type: IPFSDownloadType.Gateway, label: 'Public Gateway' },
  { type: IPFSDownloadType.Node, label: 'IPFS node' },
];

export const handleFileDownload = async (
  link: string,
  resolver: IPFSDownloadType = IPFSDownloadType.Gateway,
  vaultFileMetadata: VaultFile | undefined,
  { onCompleted, onProgress, onError }: HandleFileDownloadCallbacks,
) => {
  try {
    const { type, cid } = await parseDownloadInput(link);

    let request: Promise<Blob>;

    switch (resolver) {
      default:
      case IPFSDownloadType.Gateway:
        request = ky(new URL(cid, IPFS_GATEWAY), {
          onDownloadProgress: ({ percent }) => onProgress?.(percent),
          timeout: false,
        }).blob();
        break;
      case IPFSDownloadType.Node:
        request = queryIPFS(cid);
        break;
    }

    request
      .then(async (blob) => {
        let file = reconstructVaultFile(blob, vaultFileMetadata);

        if (type === DownloadInputType.CID) {
          onCompleted(file);
          return;
        } else {
          const params = new URLSearchParams(new URL(link).search);

          const storeType = params.get(
            AxiomLinkQuery.StoreType,
          ) as StoreType | null;
          const iv = params.get(AxiomLinkQuery.IV) as string | null;
          const key = params.get(AxiomLinkQuery.EncryptionKey) as string | null;

          if (storeType) {
            if (storeType === StoreType.WRAPPED) {
              file = await unwrapFile(file);
            }

            if (iv && key) {
              const parsedIv = fromBase64<string>(iv);
              const parsedKey = fromBase64<JsonWebKey>(key);

              const decryptedFile = await decryptFile({
                data: file,
                iv: parsedIv,
                key: parsedKey,
              });

              onCompleted(decryptedFile);
            } else {
              onCompleted(file);
            }
          } else {
            onError?.();
          }
        }
      })
      .catch(onError);
  } catch (_) {
    onError?.();
  }
};
