import { NextPage } from 'next';
import { PortalContent } from '../app/components/layout/Portal/PortalContent';
import vault, { VaultFileObjectKey } from '../app/utilities/vault';
import { Table } from '../app/components/layout/Table/Table';
import React, { useEffect, useState } from 'react';
import { useRouter } from 'next/router';
import { getPageTitle } from '../app/utilities/helpers';
import { ConfirmationModal } from '../app/components/layout/Modal/ConfirmationModal';
import { useModal } from '../app/hooks/useModal';
import {
  DownloadTypeToggle,
  useDownloadType,
} from '../app/components/DownloadTypeToggle';
import { CurrentDownloadsContextProvider } from '../app/context/CurrentDownloadsContext';
import { VaultTableRow } from '../app/components/VaultTableRow';

const Vault: NextPage = () => {
  const router = useRouter();
  const [shouldRender, setShouldRender] = useState(false);

  const { downloadType, setDownloadType } = useDownloadType();

  const {
    Modal: ConfirmClearVaultModal,
    open,
    close,
  } = useModal(ConfirmationModal, {
    title: 'Are you sure?',
    message: "You won't be able to access your files anymore.",
    actions: {
      onConfirm: () => {
        vault.clearFiles();
        close();
        router.reload();
      },
      onCancel: () => close(),
    },
  });

  useEffect(() => setShouldRender(true), []);

  return (
    <CurrentDownloadsContextProvider>
      <PortalContent
        title={getPageTitle('My Vault')}
        subtitle="Here are all your files uploaded in this browser (and session)"
      >
        <div className="flex flex-col md:flex-row space-y-4 justify-between mb-6">
          <div className="block">
            <DownloadTypeToggle
              downloadType={downloadType}
              setDownloadType={setDownloadType}
            />
          </div>
          <div className="btn-group">
            <button
              className="btn btn-sm btn-error btn-outline md:btn-md"
              onClick={() => open()}
            >
              Clear Vault
            </button>
          </div>
        </div>

        {shouldRender && (
          <Table
            className="table-compact"
            head={['', 'Filename', 'Store Type', 'Encrypted', '']}
            items={vault.getFiles}
            emptyPlaceholder="There are no files."
            render={(file, index, realIndex) => (
              <VaultTableRow
                key={`${file[VaultFileObjectKey.CID]}{${realIndex}`}
                file={file}
                index={index}
                realIndex={realIndex}
                downloadType={downloadType}
              />
            )}
          />
        )}
      </PortalContent>
      <ConfirmClearVaultModal />
    </CurrentDownloadsContextProvider>
  );
};

export default Vault;
